package org.apache.commons.lang3.tuple;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.Serializable;
import java.util.Objects;
import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.tuple.ImmutableTriple;
import org.apache.commons.lang3.tuple.Triple;

public abstract class Triple<L, M, R> implements Comparable<Triple<L, M, R>>, Serializable {
  private static final long serialVersionUID = 1L;
  
  public static final Triple<?, ?, ?>[] EMPTY_ARRAY = (Triple<?, ?, ?>[])new TripleAdapter[0];
  
  public static <L, M, R> Triple<L, M, R>[] emptyArray() {
    return (Triple<L, M, R>[])EMPTY_ARRAY;
  }
  
  public static <L, M, R> Triple<L, M, R> of(L left, M middle, R right) {
    return (Triple<L, M, R>)new ImmutableTriple(left, middle, right);
  }
  
  public int compareTo(Triple<L, M, R> other) {
    return (new CompareToBuilder()).append(getLeft(), other.getLeft())
      .append(getMiddle(), other.getMiddle())
      .append(getRight(), other.getRight()).toComparison();
  }
  
  public boolean equals(Object obj) {
    if (obj == this)
      return true; 
    if (obj instanceof Triple) {
      Triple<?, ?, ?> other = (Triple<?, ?, ?>)obj;
      return (Objects.equals(getLeft(), other.getLeft()) && 
        Objects.equals(getMiddle(), other.getMiddle()) && 
        Objects.equals(getRight(), other.getRight()));
    } 
    return false;
  }
  
  public int hashCode() {
    return Objects.hashCode(getLeft()) ^ Objects.hashCode(getMiddle()) ^ Objects.hashCode(getRight());
  }
  
  public String toString() {
    return "(" + getLeft() + "," + getMiddle() + "," + getRight() + ")";
  }
  
  public String toString(String format) {
    return String.format(format, new Object[] { getLeft(), getMiddle(), getRight() });
  }
  
  public abstract L getLeft();
  
  public abstract M getMiddle();
  
  public abstract R getRight();
}
